==> FHLBBS-4.1.tgz: wbbs/reg.c <==
#include "cgic.h"
#include "wbbs.c"

char *crypt() ;

/* html form post variables
   id  :USER
   pass:PASSWORD1
   confirm-pass:PASSWORD2
   alia:ALIAS
   name:NAME
   addr:ADDRESS
   mail:EMAIL
   yy  :YY
   mm  :MM
   dd  :DD
   sex :SEX 

struct userec
{
  char userid[IDLEN + 1];
  char realname[20];
  char username[24];
  char passwd[PASSLEN];
  uschar uflag;
  usint userlevel;
  ushort numlogins;
  ushort numposts;
  time_t firstlogin;
  time_t lastlogin;
  char lasthost[16];
  char remoteuser[8];
  char email[50];
  char address[50];
  char justify[REGLEN + 1];
  uschar month;
  uschar day;
  ushort year;
  uschar sex;
  uschar state;
};
typedef struct userec userec;

*/

static char pwbuf[14];
/********** ΨӲͱKXƵ{ ***/
char *
genpasswd(pw)
  char *pw;
{
  if (pw[0])
  {
    char saltc[2], c;
    int i;

    i = 9 * getpid();
    saltc[0] = i & 077;
    saltc[1] = (i >> 6) & 077;

    for (i = 0; i < 2; i++)
    {
      c = saltc[i] + '.';
      if (c > '9')
        c += 7;
      if (c > 'Z')
        c += 6;
      saltc[i] = c;
    }
    strcpy(pwbuf, pw);
    return crypt(pwbuf, saltc);
  }
  return "";
}

/********** ˬdnKX ***/ 
static int
bad_user_id(userid)
  char *userid;
{
  register char ch;

  if (strlen(userid) < 2)
    return 1;

  //if (not_alpha(*userid))
  //  return 1;

  if (!ci_strcmp(userid, "new"))
    return 1;

  //while (ch = *(++userid))
  //{
  //  if (not_alnum(ch))
  //    return 1;
  //}
  return 0;
}


int cgiMain() {
    char userid[IDLEN+1];
    char password[PASSLEN+1],password1[PASSLEN+1],password2[PASSLEN+1];
    char usernamex[25], realnamex[21], addressx[51];
    char emailx[51], yyx[5], mmx[3], ddx[3], sexx[2];
    char genbuf[256];
    int g_err;
    char remotehost[16],remoteuser[8];
    int gotomode;
    userec newuser;
    int allocid,i;
    time_t now;
    FILE *fp;
    char *pt1,*pt2;
    int ptmode;
    g_err=0;

   
    /**********************
    fprintf(cgiOut, "<html> <head><title>Register</title></head>\n");
    fprintf(cgiOut, "<body text=\"#200020\" link=\"#ff0000\" alink=\"#0000ff\" vlink=\"#800080\" BGCOLOR=\"#D0FFB5\">\n");
    fprintf(cgiOut, "<font color=\"#ff0000\" size=4>User List(default)</font>\n");
    fprintf(cgiOut,"<p>");
    *******************/
    /* i=list_user(); */
    now = time(0);

    allocid = getnewuserid();

    /* strcpy(userid,"ysb1");
    strcpy(password1,"yang");
    password1[8] = '\0';
    strcpy(usernamex,"username");
    strcpy(realnamex,"realname");
    strcpy(addressx,"addressasgdszhfsahreahr");
    strcpy(emailx,"esgt@eatg.aetea.tes");
    strcpy(yyx,"1999");
    strcpy(mmx,"11");
    strcpy(ddx,"11");
    strcpy(sexx,"0"); */
    strncpy(remotehost,cgiRemoteHost,15);    
    if (remotehost[0]==0)
      strncpy(remotehost,cgiRemoteAddr,15); 
    strncpy(remoteuser,cgiRemoteUser,7);
    remoteuser[7]=0;
    remotehost[15]=0;
    cgiFormStringNoNewlines("USER",userid,IDLEN+1);
    cgiFormStringNoNewlines("PASSWORD1",password1,PASSLEN+1);
    cgiFormStringNoNewlines("PASSWORD2",password2,PASSLEN+1);
    cgiFormStringNoNewlines("ALIAS",usernamex,25);
    cgiFormStringNoNewlines("NAME",realnamex,21);
    cgiFormStringNoNewlines("ADDRESS",addressx,51);
    cgiFormStringNoNewlines("EMAIL",emailx,51);
    cgiFormStringNoNewlines("YY",yyx,5);
    cgiFormStringNoNewlines("MM",mmx,3);
    cgiFormStringNoNewlines("DD",ddx,3);
    cgiFormStringNoNewlines("SEX",sexx,2);

    strncpy(newuser.userid, userid, IDLEN);
    strncpy(newuser.passwd, genpasswd(password1), 14 );
    strncpy(newuser.username, usernamex, 24);
    strncpy(newuser.realname, realnamex, 20);
    strncpy(newuser.address, addressx, 50);
    strncpy(newuser.email, emailx, 50);
    newuser.year=(ushort)atoi(yyx);
    newuser.month=(uschar)atoi(mmx);
    newuser.day=(uschar)atoi(ddx);
    newuser.sex=(uschar)(atoi(sexx)-1); 

    newuser.userlevel = PERM_DEFAULT;
    newuser.numlogins=0;
    newuser.numposts=0;
    newuser.uflag=COLOR_FLAG | BRDSORT_FLAG | MOVIE_FLAG;
    newuser.firstlogin = newuser.lastlogin = time(NULL);

    /*ˬdJ*/
    if (bad_user_id(newuser.userid)){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut,"LkoӥNAШϥέ^rAåBn]tŮ\n");
       g_err=1;
    }
    else if(searchuser(newuser.userid)){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut,"NwgHϥ\n");
       g_err=1;
    }
    if((strlen(newuser.passwd) < 3) || (!strcmp(newuser.passwd, newuser.userid))){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut,"KX²ADJIAܤ֭n 4 ӦrAЭsJ\n");
       g_err=1;
    }
    if(strlen(newuser.realname) < 4){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut,"DzmW֩ӦrH\n");
       g_err=1;
    }
    if(strlen(newuser.address) < 8){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut,"Sou}aH\n");
       g_err=1;
    }
    if(!strchr(newuser.email, '@') || strlen(newuser.email)==0){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut," FzvqAжguꪺ E-mail addressA"
       " HT{դUA\n");
       g_err=1;
    }
    if(newuser.month > 12 || newuser.month < 1 
      || newuser.day > 31 || newuser.day < 1 || newuser.year<1910 || 
      newuser.year>2005){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut,"ͤFI\n");
       g_err=1;
    }
    if(g_err==1){ 
       fprintf(cgiOut,"</font>\n");
       fprintf(cgiOut,"</body></html>\n");
       exit(0);
    }
    /**** gJU ***/
    if(substitute_record(fn_passwd, &newuser, sizeof(newuser), allocid) == -1)
    {
        cgiHeaderContentType("text/html");
       fprintf(cgiOut," UƼgJ!!!\n");
       fprintf(cgiOut,"</font>\n");
       fprintf(cgiOut,"</body></html>\n");
       exit(0);
    }
    resolve_ucache();
    setuserid(allocid, newuser.userid);
    memcpy(&cuser,&newuser,sizeof(newuser));//ƻsƥHK쥿T
/**************************************
    fprintf(cgiOut," U\\n");
    fprintf(cgiOut,"</font>\n");
    fprintf(cgiOut,"</body></html>\n");
******************************************/
    /*** U᪽nJ ****/
    /*** nJ\ ***/
    cgiFormInteger("GOTO",&gotomode,0);
  	set_idcookie(1,newuser.userid,newuser.passwd);
	/********  OU ************/
        sprintf(genbuf,"%4d  %s@%s",1,remoteuser,remotehost); 
        log_usies("ENTER", genbuf);
        cgiHeaderContentType("text/html");
		sethomepath(genbuf, newuser.userid);
        mkdir(genbuf, 0755);
        sethomedir(genbuf,newuser.userid);
        close(open(genbuf, O_RDWR|O_CREAT,0600));
        if (gotomode)
  	   {
	     if ((fp=fopen(DEFDIR FHLWELCOME,"r"))==NULL)
	     {
	      fprintf(cgiOut,"<h1>HRwzI</h1>");
		  return(0);
	     }
	   }
	   else
	   {
          if ((fp=fopen(DEFDIR MAINFUNC,"r"))==NULL)
          {
            fprintf(cgiOut,"<h1>D\\}ҿ~I</h1><p>\n");
            return(0);
          }
	   }
        while (fgets(genbuf,255,fp))
        {
          pt1=genbuf;
          while (pt2=strstr(pt1,"<!--$"))
          {
           *pt2=0;
           pt2=pt2+5;
           fprintf(cgiOut,"%s", pt1);/* LXtokener */
           pt1=strstr(pt2,"-->");
           if (pt1==NULL) 
           {
              fprintf(cgiOut,"榡ɿ~IP@椺䤣 -->");
              return 0;
           }
           *pt1=0;
           pt1=pt1+3;
           ptmode=atoi(pt2);
           if (ptmode)   SSI(ptmode);
          }
           fprintf(cgiOut,"%s",pt1);
        }

    exit(0);
}

int
getnewuserid()
{
    userec utmp;
    int i,fd;
    
    if((fd = open(fn_passwd,O_RDWR|O_CREAT,0666)) == -1) {
        return -1;
    }

    i = 0;
    while(read(fd,&utmp,sizeof(utmp)) == sizeof(utmp)){
       i++;
       if(utmp.userid[0] == '\0') break;
    }
    close(fd) ;
    return i;    
}    

int list_user() {
    userec utmp;
    int i, fd;
    
    if((fd = open(fn_passwd,O_RDWR|O_CREAT,0666)) == -1) {
        return ;
    }

    i = 0;
    fprintf(cgiOut,"<p>");
    while(read(fd,&utmp,sizeof(utmp)) == sizeof(utmp)){
       i++;
       fprintf(cgiOut,"count=%d userid=%s|| pass=%s||<p>",i,utmp.userid,utmp.passwd);
    }
    close(fd) ;
    return 0;
}

==> FHLBBS-4.2.tgz: wbbs/reg.c <==
#include "cgic.h"
#include "wbbs.c"

char *crypt() ;

/* html form post variables
   id  :USER
   pass:PASSWORD1
   confirm-pass:PASSWORD2
   alia:ALIAS
   name:NAME
   addr:ADDRESS
   mail:EMAIL
   yy  :YY
   mm  :MM
   dd  :DD
   sex :SEX 

struct userec
{
  char userid[IDLEN + 1];
  char realname[20];
  char username[24];
  char passwd[PASSLEN];
  uschar uflag;
  usint userlevel;
  ushort numlogins;
  ushort numposts;
  time_t firstlogin;
  time_t lastlogin;
  char lasthost[16];
  char remoteuser[8];
  char email[50];
  char address[50];
  char justify[REGLEN + 1];
  uschar month;
  uschar day;
  ushort year;
  uschar sex;
  uschar state;
};
typedef struct userec userec;

*/

static char pwbuf[14];
/********** ΨӲͱKXƵ{ ***/
char *
genpasswd(pw)
  char *pw;
{
  if (pw[0])
  {
    char saltc[2], c;
    int i;

    i = 9 * getpid();
    saltc[0] = i & 077;
    saltc[1] = (i >> 6) & 077;

    for (i = 0; i < 2; i++)
    {
      c = saltc[i] + '.';
      if (c > '9')
        c += 7;
      if (c > 'Z')
        c += 6;
      saltc[i] = c;
    }
    strcpy(pwbuf, pw);
    return crypt(pwbuf, saltc);
  }
  return "";
}

/********** ˬdnKX ***/ 
int
bad_user_id(userid)
  char *userid;
{
  register char ch;

  if (strlen(userid) < 2)
    return 1;

  if (not_alpha(*userid))
    return 1;

  if (!ci_strcmp(userid, "new"))
    return 1;

  while (ch = *(++userid))
  {
    if (not_alnum(ch))
      return 1;
  }
  return 0;
}


int cgiMain() {
    char userid[IDLEN+1];
    char password[PASSLEN+1],password1[PASSLEN+1],password2[PASSLEN+1];
    char usernamex[25], realnamex[21], addressx[51];
    char emailx[51], yyx[5], mmx[3], ddx[3], sexx[2];
    char genbuf[256];
    int g_err;
    char remotehost[16],remoteuser[8];
    int gotomode;
    userec newuser;
    int allocid,i;
    time_t now;
    FILE *fp;
    char *pt1,*pt2;
    int ptmode;
    g_err=0;

   
    /**********************
    fprintf(cgiOut, "<html> <head><title>Register</title></head>\n");
    fprintf(cgiOut, "<body text=\"#200020\" link=\"#ff0000\" alink=\"#0000ff\" vlink=\"#800080\" BGCOLOR=\"#D0FFB5\">\n");
    fprintf(cgiOut, "<font color=\"#ff0000\" size=4>User List(default)</font>\n");
    fprintf(cgiOut,"<p>");
    *******************/
    /* i=list_user(); */
    now = time(0);

    allocid = getnewuserid();

    /* strcpy(userid,"ysb1");
    strcpy(password1,"yang");
    password1[8] = '\0';
    strcpy(usernamex,"username");
    strcpy(realnamex,"realname");
    strcpy(addressx,"addressasgdszhfsahreahr");
    strcpy(emailx,"esgt@eatg.aetea.tes");
    strcpy(yyx,"1999");
    strcpy(mmx,"11");
    strcpy(ddx,"11");
    strcpy(sexx,"0"); */
    strncpy(remotehost,cgiRemoteHost,15);    
    if (remotehost[0]==0)
      strncpy(remotehost,cgiRemoteAddr,15); 
    strncpy(remoteuser,cgiRemoteUser,7);
    remoteuser[7]=0;
    remotehost[15]=0;
    cgiFormStringNoNewlines("USER",userid,IDLEN+1);
    cgiFormStringNoNewlines("PASSWORD1",password1,PASSLEN+1);
    cgiFormStringNoNewlines("PASSWORD2",password2,PASSLEN+1);
    cgiFormStringNoNewlines("ALIAS",usernamex,25);
    cgiFormStringNoNewlines("NAME",realnamex,21);
    cgiFormStringNoNewlines("ADDRESS",addressx,51);
    cgiFormStringNoNewlines("EMAIL",emailx,51);
    cgiFormStringNoNewlines("YY",yyx,5);
    cgiFormStringNoNewlines("MM",mmx,3);
    cgiFormStringNoNewlines("DD",ddx,3);
    cgiFormStringNoNewlines("SEX",sexx,2);

    strncpy(newuser.userid, userid, IDLEN);
    strncpy(newuser.passwd, genpasswd(password1), 14 );
    strncpy(newuser.username, usernamex, 24);
    strncpy(newuser.realname, realnamex, 20);
    strncpy(newuser.address, addressx, 50);
    strncpy(newuser.email, emailx, 50);
    newuser.year=(ushort)atoi(yyx);
    newuser.month=(uschar)atoi(mmx);
    newuser.day=(uschar)atoi(ddx);
    newuser.sex=(uschar)(atoi(sexx)-1); 

    newuser.userlevel = PERM_DEFAULT;
    newuser.numlogins=0;
    newuser.numposts=0;
    newuser.uflag=COLOR_FLAG | BRDSORT_FLAG | MOVIE_FLAG;
    newuser.firstlogin = newuser.lastlogin = time(NULL);

    /*ˬdJ*/
    if (bad_user_id(newuser.userid)){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut,"LkoӥNAШϥέ^rAåBn]tŮ\n");
       g_err=1;
    }
    else if(searchuser(newuser.userid)){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut,"NwgHϥ\n");
       g_err=1;
    }
    if((strlen(newuser.passwd) < 3) || (!strcmp(newuser.passwd, newuser.userid))){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut,"KX²ADJIAܤ֭n 4 ӦrAЭsJ\n");
       g_err=1;
    }
    if(strlen(newuser.realname) < 4){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut,"DzmW֩ӦrH\n");
       g_err=1;
    }
    if(strlen(newuser.address) < 8){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut,"Sou}aH\n");
       g_err=1;
    }
    if(!strchr(newuser.email, '@') || strlen(newuser.email)==0){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut," FzvqAжguꪺ E-mail addressA"
       " HT{դUA\n");
       g_err=1;
    }
    if(newuser.month > 12 || newuser.month < 1 
      || newuser.day > 31 || newuser.day < 1 || newuser.year<1910 || 
      newuser.year>2005){
        cgiHeaderContentType("text/html");
       fprintf(cgiOut,"ͤFI\n");
       g_err=1;
    }
    if(g_err==1){ 
       fprintf(cgiOut,"</font>\n");
       fprintf(cgiOut,"</body></html>\n");
       exit(0);
    }
    /**** gJU ***/
    if(substitute_record(fn_passwd, &newuser, sizeof(newuser), allocid) == -1)
    {
        cgiHeaderContentType("text/html");
       fprintf(cgiOut," UƼgJ!!!\n");
       fprintf(cgiOut,"</font>\n");
       fprintf(cgiOut,"</body></html>\n");
       exit(0);
    }
    resolve_ucache();
    setuserid(allocid, newuser.userid);
    memcpy(&cuser,&newuser,sizeof(newuser));//ƻsƥHK쥿T
/**************************************
    fprintf(cgiOut," U\\n");
    fprintf(cgiOut,"</font>\n");
    fprintf(cgiOut,"</body></html>\n");
******************************************/
    /*** U᪽nJ ****/
    /*** nJ\ ***/
    cgiFormInteger("GOTO",&gotomode,0);
  	set_idcookie(1,newuser.userid,newuser.passwd);
	/********  OU ************/
        sprintf(genbuf,"%4d  %s@%s",1,remoteuser,remotehost); 
        log_usies("ENTER", genbuf);
        cgiHeaderContentType("text/html");
		sethomepath(genbuf, newuser.userid);
        mkdir(genbuf, 0755);
        sethomedir(genbuf,newuser.userid);
        close(open(genbuf, O_RDWR|O_CREAT,0600));
        if (gotomode)
  	   {
	     if ((fp=fopen(DEFDIR FHLWELCOME,"r"))==NULL)
	     {
	      fprintf(cgiOut,"<h1>HRwzI</h1>");
		  return(0);
	     }
	   }
	   else
	   {
          if ((fp=fopen(DEFDIR MAINFUNC,"r"))==NULL)
          {
            fprintf(cgiOut,"<h1>D\\}ҿ~I</h1><p>\n");
            return(0);
          }
	   }
        while (fgets(genbuf,255,fp))
        {
          pt1=genbuf;
          while (pt2=strstr(pt1,"<!--$"))
          {
           *pt2=0;
           pt2=pt2+5;
           fprintf(cgiOut,"%s", pt1);/* LXtokener */
           pt1=strstr(pt2,"-->");
           if (pt1==NULL) 
           {
              fprintf(cgiOut,"榡ɿ~IP@椺䤣 -->");
              return 0;
           }
           *pt1=0;
           pt1=pt1+3;
           ptmode=atoi(pt2);
           if (ptmode)   SSI(ptmode);
          }
           fprintf(cgiOut,"%s",pt1);
        }

    exit(0);
}

int
getnewuserid()
{
    userec utmp;
    int i,fd;
    
    if((fd = open(fn_passwd,O_RDWR|O_CREAT,0666)) == -1) {
        return -1;
    }

    i = 0;
    while(read(fd,&utmp,sizeof(utmp)) == sizeof(utmp)){
       i++;
       if(utmp.userid[0] == '\0') break;
    }
    close(fd) ;
    return i;    
}    

int list_user() {
    userec utmp;
    int i, fd;
    
    if((fd = open(fn_passwd,O_RDWR|O_CREAT,0666)) == -1) {
        return ;
    }

    i = 0;
    fprintf(cgiOut,"<p>");
    while(read(fd,&utmp,sizeof(utmp)) == sizeof(utmp)){
       i++;
       fprintf(cgiOut,"count=%d userid=%s|| pass=%s||<p>",i,utmp.userid,utmp.passwd);
    }
    close(fd) ;
    return 0;
}
